function [beta, opt, betaci, bestci] = fluofit(data, em, ex, opt)
%FLUOFIT Fitting of fluorescence spectra.
%   [BETA, OPT] = FLUOFIT(DATA, EM, EX, OPT) fits the fluorescence spectra
%   in DATA (with emission wavelengths EM in rows and excitation wavelengths
%   EX in columns) with asymetric log-normal functions. OPT is an
%   option structure generated with FLUOFITOPT to influence the fitting.
%
%   EXAMPLE:
%      [beta, opt] = fluofit(data, em, ex, fluofitopt('iterations', 100));
%
%   AUTHOR:
%      Wouter Caarls, MPIbpc <wouter@caarls.org>

    if nargin < 4
        opt = fluofitopt();
    end

    if size(em, 1) > size(em, 2)
        em = em';
    end
    
    md = max(max(max(data)));
    data = data./md;
    
    if opt.wavenumber
        desc.ex = 10^7./ex;
        desc.em = 10^7./em;
    else
        desc.ex = ex;
        desc.em = em;
    end
    
    if opt.global
        desc.reps = size(data, 3);
        reps = 1;
        beta = [];
        betaci = [];
    else
        desc.reps = 1;
        reps = size(data, 3);
        beta = {};
        betaci = {};
    end
    
    for r = 1:reps
        bestr = -1000;
        bestbeta = [];
        bestci = [];
        
        if opt.global
            desc.data = data;
            y = reshape(permute(data, [1 2 3]), 1, prod(size(data))); % CHECK
        else
            repdata = data(:,:,r);
            desc.data = repdata;
            y = reshape(permute(repdata, [1 2]), 1, prod(size(repdata))); % CHECK
        end
        x = 1:length(y);
        
        desc.zeros = (y<=0);
        
        for i = 1:opt.iterations
            [modelfun, splitfun, opt] = fluofitmdl(opt, desc);
            [beta1, r, j] = nlinfit(x, y, modelfun, opt.beta0, statset('MaxIter', 100, 'DerivStep', 0.005));
            ci = nlparci(beta1, r, 'jacobian', j);
            
            % 03-03-2016: Change in nlinfit behavior
            r = reshape(r, 1, numel(r));

            rq = rsq(y, y-r);
            
            fprintf('R^2 = %f\n', rq);
           
            if rq > bestr
                bestr = rq;
                bestbeta = beta1;
                bestci = ci;
                [ms, nxh] = splitfun(bestbeta);
                
                clf
                fluofitplot([ms nxh], opt, desc);
                drawnow;
                
                if bestr > 0.9999
                    break;
                end
            end
        end

        [ms, nxh, xs, xh] = splitfun(bestbeta);
        beta = [beta [ms nxh.*md]];
        
        [lms, lnxh, lxs, lxh] = splitfun(bestci(:,1)');
        [rms, rnxh, rxs, rxh] = splitfun(bestci(:,2)');
        betaci = [betaci [abs(lms-rms)/2 (abs(lnxh-rnxh)/2).*md]];
    end
