function fluofitexport(data, em, ex, beta, base, opt)
%FLUOFITEXPORT Exports fitted fluorescence spectra to .csv.
%   FLUOFITEXPORT(DATA, EM, EX, BETA, BASE, OPT) exports the fitted
%   fluorescence spectra in DATA, EM, EX, BETA to BASE.csv in
%   comma-separated values format. OPT is the options structure returned
%   by FLUOFIT.
%
%   EXAMPLE:
%      fluofitexport(data, em, ex, beta, 'myfit', opt);
%
%   AUTHOR:
%      Wouter Caarls, MPIbpc <wouter@caarls.org>

    desc = fluofitdesc(data, em, ex, opt);

    desc.exs = length(desc.ex);
    desc.ems = length(desc.em);

    if opt.wavenumber
        em = 10^7./desc.em;
        ex = 10^7./desc.ex;
    else
        em = desc.em;
        ex = desc.ex;
    end
    
    if length(beta) == 1
        beta = beta{1};
    end
    hs = opt.bands*3;
    or = (length(beta)-hs)/desc.reps;

    tval = cell(desc.reps, desc.exs);
    tfit = cell(desc.reps, desc.exs);
    tres = cell(desc.reps, desc.exs);
    bfit = cell(desc.reps, desc.exs, opt.bands);

    for r = 1:desc.reps
        for i = 1:desc.exs
            te = zeros(1, desc.ems);
            for b = 1:opt.bands
                be = alogn(beta((b-1)*3+1), beta((b-1)*3+2), beta((b-1)*3+3), beta(hs+(r-1)*or+(b-1)*desc.exs+i), desc.em);
                bfit{r, i, b} = be;
                te = te + be;
            end

            tval{r, i} = desc.data(:, i, r);
            tfit{r, i} = te;
            tres{r, i} = desc.data(:, i, r)-te';
        end
    end

    fid = fopen([base '.csv'], 'w');
    %fid = 1;
    
    for emi = 0:desc.ems
        if emi == 0
            fprintf(fid, 'em');
        else
            fprintf(fid, '%d', em(emi));
        end
        for rep = 1:desc.reps
            for exi = 1:desc.exs
                if emi == 0
                    fprintf(fid, ',rep %d ex %d,fit,residual', rep, ex(exi));
                else
                    d = tval{rep, exi};
                    f = tfit{rep, exi};
                    r = tres{rep, exi};
                    fprintf(fid, ',%d,%d,%d', d(emi), f(emi), r(emi));
                end
                
                for band = 1:opt.bands
                    if emi == -1
                        fprintf(fid, ',');
                    elseif emi == 0
                        fprintf(fid, ',band %d', band);
                    else
                        b = bfit{rep, exi, band};
                        fprintf(fid, ',%d', b(emi));
                    end
                end
            end
        end
        fprintf(fid, '\n');
    end
    
    fclose(fid);
    


