function [mdl, split, opt] = fluofitmdl(opt, desc)
%FLUOFITMDL Create a model function for fluorescence spectrum fitting.
%   [MDL, SPLIT, BETA] = FLUOFITMDL(OPT, DESC) returns a closure that
%   can be used as a model function to fit a fluorescence spectrum.
%   SPLIT splits the resulting model parameters into categories with
%   a constant interpretation. BETA can be used as an initial guess for the
%   model parameters.
%
%   AUTHOR:
%      Wouter Caarls, MPIbpc <wouter@caarls.org>

    mdl = @mdlfun;
    split = @splitbeta;
    beta = [];
    
    desc.exs = length(desc.ex);
    desc.ems = length(desc.em);
    
    n = 1;
    
    dc = {};

    emshapeparm = addparm(opt.bands*3, 'rhn');
    if opt.exfit
        if opt.fixratio
            exshapeparm = addparm((opt.bands-1)*3, 'RHN');
            exheightparm = [];
            for i = 1:desc.reps
                add = addparm(opt.bands-1, 'e');
                exheightparm = [exheightparm add];
                exheightparm = [exheightparm add(1)];
            end
            if opt.globalratio
                ratioparm = ones(1, desc.reps) * addparm(1, 'a');
            else
                ratioparm = addparm(desc.reps, 'a');
            end
        else
            ratioparm = [];
            exshapeparm = addparm(opt.bands*3, 'RHN');
            exheightparm = addparm(desc.reps*opt.bands, 'e');
        end
    else
        if opt.fixratio
            exshapeparm = [];
            exheightparm = [];
            for i = 1:desc.reps
                add = addparm(desc.exs*(opt.bands-1), 'e');
                exheightparm = [exheightparm add];
                exheightparm = [exheightparm add(1:desc.exs)];
            end
            if opt.globalratio
                ratioparm = ones(1, desc.reps) * addparm(1, 'a');
            else
                ratioparm = addparm(desc.reps, 'a');
            end
        else
            ratioparm = [];
            exshapeparm = [];
            exheightparm = addparm(desc.reps * desc.exs * opt.bands, 'e');
        end
    end
    
    if opt.defaultconstraints
        opt.constraints = [dc opt.constraints];
    end

    % Avoid duplicate constraints -- it slows down the fixing
    c = cell(1, (numel(opt.constraints)/3));
    for i = 1:numel(c)
        c{i} = opt.constraints{(i-1)*3+1};
    end
    [B, I] = unique(c, 'last');

    % 03-03-2016: Change in unique behavior
    I = reshape(I, 1, numel(I));

    take = reshape([(I-1)*3+1;(I-1)*3+2;(I-1)*3+3], 1, numel(I)*3);
    
    opt.constraints = opt.constraints(take);
    opt.beta0 = beta;
    
    function [emshape, newexheight, exshape, exheight] = splitbeta(beta)
        
        % Split parameters
        emshape = beta(emshapeparm);
        exshape = beta(exshapeparm);
        
        [emshape, exshape] = fluofitfix(emshape, exshape, opt.constraints);
        
        % Warning: may lead to N2 being outside clamp range
        if opt.exfit & opt.bands == 3 & opt.redshift
            exshape(6) = exshape(3) - abs(exshape(6)-exshape(3));
        end
        
        exheight = abs(beta(exheightparm));
        ratio = abs(beta(ratioparm));
        
        % Adjust final band height according to the ratio if necessary
        if opt.fixratio
            onerep  = length(exheight)/desc.reps;
            oneband = onerep/opt.bands;
            for i = 1:desc.reps
                exheight(((i-1)*onerep + (opt.bands-1)*oneband+1):(i*onerep)) = ...
                    ratio(i);
%                    exheight(((i-1)*onerep + (opt.bands-1)*oneband+1):(i*onerep)) * ratio(i);
            end

            if length(exshape)
                exshape = [exshape exshape(1:(length(exshape)/(opt.bands-1)))];
            end
        end
        
        % Generate excitation spectrum from model if necessary
        if opt.exfit
            newexheight = [];
            for i = 0:(desc.reps-1)
                for j = 0:(opt.bands-1)
                    newexheight = [newexheight alogn(exshape(j*3+1), exshape(j*3+2), exshape(j*3+3), exheight(i*opt.bands+j+1), desc.ex)];
                end
            end
        else
            newexheight = exheight;
        end
       
    end
    
    function y = mdlfun(beta, x)
        
        [emshape, exheight] = splitbeta(beta);
        
        % Apply the model
        y = [];
        for i = 0:(desc.reps-1)
            yr = [];
            for j = 0:(desc.exs-1)
                ye = zeros(1, desc.ems);
                for k = 0:(opt.bands-1)
                    ye = ye + alogn(emshape(k*3+1), emshape(k*3+2), emshape(k*3+3), exheight((i*opt.bands+k)*desc.exs+j+1), desc.em);
                end
                yr = [yr ye];
            end
            y = [y yr];
        end
        
        y(desc.zeros) = 0;
    end
    
    function s = addparm(l, m)
        s = n:(n+l-1);
        n = n + l;
        
        for i = 1:l
            p = floor((i-1)/length(m))+1;
            switch m(rem(i-1, length(m))+1)
                case 'r'
                    beta = [beta 0.7 + 0.2*randn()];
                    dc   = [dc {['r' num2str(p)], 0.75, 0.25}];
                case 'h'
                    beta = [beta 3000 + 1000 * randn()];
                    dc   = [dc {['h' num2str(p)], 3000, 2500}];
                case 'n'
                    beta = [beta 19000 + 1500 * randn()];
                    dc   = [dc {['n' num2str(p)], 19000, 3000}];
                case 'R'
                    beta = [beta 1.5 + 0.5*randn()];
                    dc   = [dc {['R' num2str(p)], 1.5, 0.5}];
                case 'H'
                    beta = [beta 3000 + 1000 * randn()];
                    dc   = [dc {['H' num2str(p)], 3000, 2500}];
                case 'N'
                    beta = [beta 25000 + 1000 * randn()];
                    dc   = [dc {['N' num2str(p)], 25000, 2000}];
                case 'e'
                    beta = [beta 1 + 1 * randn()];
                case 'a'
                    beta = [beta 1 + 1 * randn()];
            end
        end
    end
end
